﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.IO;


namespace XMLVerify {
    /// <summary>
    /// Interaction logic for MyFileList2.xaml
    /// </summary>
    public partial class MyFileList2 : UserControl {
        String path;
        SingVar singvar;
        Window parent;
        private FileSystemWatcher fsw;

        private bool browserVisible = true;

        public MyFileList2() {
            InitializeComponent();
        }

        public void init(Window parent) {

            CModell instance = CModell.getInstance();
            this.path = instance.ActualDirectory;
            if (!path.EndsWith("\\")) path += "\\";

            this.parent = parent;

            this.path = instance.ActualDirectory;
            if (!path.EndsWith("\\")) path += "\\";

            fsw = new FileSystemWatcher();
            fsw.EnableRaisingEvents = false;
            // Events definieren
            fsw.Changed += new FileSystemEventHandler(FSW_Changed);
            fsw.Created += new FileSystemEventHandler(FSW_Changed);
            fsw.Deleted += new FileSystemEventHandler(FSW_Changed);
            fsw.Renamed += new RenamedEventHandler(FSW_Changed);

            setFileWatcher(path);

            //lPath = new Label();
            lPath.Content = "Path";

            createFileList();
            insertLogicalDrives();
        }

        delegate void showFilesInListViewCallback(string actualPath);

        void FSW_Changed(object sender, FileSystemEventArgs e) {
            Application.Current.Dispatcher.Invoke(new Action(() => { createFileList(); }));
        }

        private void setFileWatcher(String actualPath) {
            if (actualPath.Length > 0) {
                fsw.Path = actualPath;
                fsw.Filter = "*.*";
                fsw.EnableRaisingEvents = true;
            }
        }  // setFileWatcher




        public void updatePath() {
            path = CModell.getInstance().ActualDirectory;
            if (!path.EndsWith("\\")) path += "\\";
            createFileList();
            setFileWatcher(path);
        }

        public void update() {
            createFileList();
        }


        public void setFontsize(int size) {
            listbox.FontSize = size;
            comboBox1.FontSize = size;
            lPath.FontSize = size;
            laufwerk.FontSize = size;
        }

        public void setBrowserVisible(bool visible) {
            browserVisible = visible;
            if (visible) {
                grid.RowDefinitions[3].Height = new GridLength(1, GridUnitType.Star);
            }
            else {
                grid.RowDefinitions[3].Height = new GridLength(0);
            }
        }

        private void listbox_MouseDoubleClick(object sender, MouseButtonEventArgs e) {
            FileObject fobj = (FileObject)listbox.SelectedItem;
            if (fobj.isParent) {
                DirectoryInfo dir = Directory.GetParent(path);
                path = dir.Parent.FullName;
                if (!path.EndsWith("\\")) path += "\\";
                createFileList();
            }
            else if (fobj.isDir) {
                path += fobj.name + "\\";
                createFileList();
            }
            else {
                CModell modell = CModell.getInstance();
                modell.checkFile1();
                modell.checkFile2();
                modell.FilenameXML = path + fobj.name;                
                modell.notifyObservers();
            }
        }

        private void listbox_KeyDown(object sender, KeyEventArgs e) {
            if (Keyboard.IsKeyDown(Key.Delete)) {
                deletFile();
                e.Handled = true;
                return;
            }
            if (Keyboard.IsKeyDown(Key.F2)) {
                renameFileF2();
                e.Handled = true;
                return;
            }
            if (Keyboard.IsKeyDown(Key.Insert)) {
                newFolder();
                e.Handled = true;
                return;
            }
        }

        private void newFolder() {
            if (!path.Equals("")) {
                if (!path.EndsWith("\\")) path += "\\";
                FReadString dialog = new FReadString(parent, "Neuer Ordner", "Ordnername");
                dialog.ShowDialog();
                if (dialog.Retcode) {
                    String newDir = path + dialog.tInput.Text.Trim();
                    System.IO.Directory.CreateDirectory(newDir);
                    //createFileList();
                }
            }
        } // NewFolder




        private void deletFile() {
            int i = listbox.SelectedIndex;
            if (i >= 0) {
                FileObject fobj = (FileObject)listbox.SelectedItem;
                if (Basis.readYesNoCancel("Do you really want to delete this file(s)?\r\n" + fobj.name, "Delete") == MessageBoxResult.Yes) {
                    String filename = path + fobj.name;
                    if (File.Exists(filename)) {
                        File.Delete(filename);
                        filename = Basis.changeFilenameExt(path + fobj.name, "css"); ;
                        if (File.Exists(filename)) {
                            File.Delete(filename);
                        }
                        //createFileList();
                    }
                    if (Directory.Exists(filename)) {
                        Directory.Delete(filename);
                        //createFileList();
                    }

                }
            }
        }

        public void renameFileF2() {
            int i = listbox.SelectedIndex;
            if (i >= 0) {
                FileObject fobj = (FileObject)listbox.SelectedItem;
                String fullFilename = path + fobj.name;
                String filename;
                Basis.splitFilename(fullFilename, out path, out filename);
                FRenameWindow dialog = new FRenameWindow(parent, filename, false);
                dialog.ShowDialog();
                if (dialog.Retcode) {
                    String newfile = dialog.tBis.Text.Trim();
                    String newFilename;
                    if (newfile.IndexOf("\\") >= 0)
                        newFilename = newfile;
                    else
                        newFilename = path + newfile;
                    System.IO.File.Move(fullFilename, newFilename);
                    //createFileList();
                }
            }
        }  // Rename


        // string[] dirs = Directory.GetFiles(@"c:\", "c*");
        private void createFileList() {
            listbox.Items.Clear();
            // nicht im root
            if (path.Length > 3) {
                listbox.Items.Add(new FileObject("..", false, true));
            }

            lPath.Content = "Akt. Dir: " + Basis.getLastPath(path);
            lPath.ToolTip = path;

            string[] subdirectoryEntries = Directory.GetDirectories(path);
            foreach (string fileName in subdirectoryEntries) {
                listbox.Items.Add(new FileObject(Basis.getFilename(fileName), true, false));
            }
            string[] fileEntries = Directory.GetFiles(path, "*.xml");
            foreach (string fileName in fileEntries) {
                listbox.Items.Add(new FileObject(Basis.getFilename(fileName), false, false));
            }

            listbox.AlternationCount = 2;

        } // createFileList


        void insertLogicalDrives() {
            try {
                comboBox1.Items.Clear();
                string[] drives = System.IO.Directory.GetLogicalDrives();

                foreach (string str in drives) {
                    comboBox1.Items.Add(str);
                }
                comboBox1.Items.Add("Desktop");
                comboBox1.Items.Add("Meine Dokumente");
                comboBox1.Items.Add("Downloads");
            }
            catch (System.IO.IOException) {
                System.Console.WriteLine("An I/O error occurs.");
            }
            catch (System.Security.SecurityException) {
                System.Console.WriteLine("The caller does not have the required permission.");
            }
        }


        private void comboBox1_SelectionChanged(object sender, SelectionChangedEventArgs e) {
            String sLw = (String)comboBox1.SelectedItem;
            if (sLw.Equals("Desktop")) {
                path = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory);
            }
            else if (sLw.Equals("Meine Dokumente")) {
                path = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            }
            else if (sLw.Equals("Downloads")) {
                path = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory);
                path.Replace("Desktop", "Downloads");
            }
            else {
                path = sLw;
            }
            if (!path.EndsWith("\\")) path += "\\";
            createFileList();
        }


    }
}
