﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.IO;
using System.Runtime.InteropServices;
// using System.Drawing;
using System.Xml;

// neu in WPF
using System.Windows.Controls;
using System.Windows;
using System.Windows.Input;


namespace XMLVerify {
    class MyTextBox : DockPanel, IView {

        private TextBox editor;
        // private Panel panelLabels;
        WrapPanel wrapPanel;
        private Label lRowNr;
        private Label lColumnNr;
        private Label lDatei;
        private int active = 0;
        private int id = 0;


        private Boolean bEditor;

        private Boolean modified = false;


        String search = "";
        Boolean ignoreCase = false;
        int position = 0;

        Boolean isXMLEditor = false;

        CModell modell = null;


        public String Search {
            set {
                search = value;
                position = 0;  // sicherheit
            }
        }
        public Boolean IgnoreCase {
            set {
                ignoreCase = value;
                position = 0;
            }
        }


        public Boolean isActive() {
            return active == id;
        }


        public Boolean Modified {
            get { return modified; }
            set { modified = value; }
        }


        public Boolean WordWrap {
            get { return editor.TextWrapping == TextWrapping.Wrap; }
            set {
                if (value)
                    editor.TextWrapping = TextWrapping.Wrap;
                else
                    editor.TextWrapping = TextWrapping.NoWrap;
            }
        }


        public String Text {
            get {
                return editor.Text;
            }
            set { editor.Text = value; }
        }


        public void setFontsize(int size) {
            editor.FontSize = size;
            if (bEditor) {
                lRowNr.FontSize = size;
                lColumnNr.FontSize = size;
                lDatei.FontSize = size;
            }
        }


        public MyTextBox(int id, Boolean bEditor, Boolean isXMLEditor) {
            this.bEditor = bEditor;
            if (bEditor)
                modell = CModell.getInstance();
            else
                modell = null;

            //this.Background = System.Windows.Media.Brushes.Red;

            this.LastChildFill = true;

            this.HorizontalAlignment = HorizontalAlignment.Stretch;
            this.VerticalAlignment = VerticalAlignment.Stretch;

            editor = new TextBox();
            this.editor.TextWrapping = TextWrapping.Wrap;
            this.editor.AcceptsReturn = true;
            this.editor.AcceptsTab = true;
            this.editor.HorizontalScrollBarVisibility = ScrollBarVisibility.Visible;
            this.editor.VerticalScrollBarVisibility = ScrollBarVisibility.Visible;

            if (bEditor) {
                modell.addObserver(this);
                this.isXMLEditor = isXMLEditor;

                wrapPanel = new WrapPanel();

                lRowNr = new Label();
                lRowNr.Content = "Zeilennr";
                lDatei = new Label();
                lDatei.Content = "Datei: ";

                lColumnNr = new Label();
                lColumnNr.Content = "Spalte";

                lRowNr.FontSize = 12;
                lColumnNr.FontSize = 12;
                lDatei.FontSize = 12;

                lRowNr.Margin = new Thickness(0, 0, 10, 0);  // left top right Bottom
                lColumnNr.Margin = new Thickness(0, 0, 10, 0);  // left top right Bottom

                wrapPanel.HorizontalAlignment = HorizontalAlignment.Left;
                wrapPanel.Children.Add(lRowNr);
                wrapPanel.Children.Add(lColumnNr);
                wrapPanel.Children.Add(lDatei);

                DockPanel dockpanel = new DockPanel();
                dockpanel.LastChildFill = true;

                DockPanel.SetDock(wrapPanel, Dock.Bottom);
                DockPanel.SetDock(this.editor, Dock.Top);
                dockpanel.Children.Add(wrapPanel);
                dockpanel.Children.Add(editor);

                this.Children.Add(dockpanel);
            }
            else {
                this.Children.Add(this.editor);

            }
            this.id = id;

            this.editor.FontSize = 12;
            this.editor.FontFamily = new System.Windows.Media.FontFamily("Courier New");

            this.editor.MaxLength = 327670;

            this.editor.LostFocus += this.textBox1_LostFocus;
            this.editor.GotFocus += this.textBox1_GotFocus;
            this.editor.TextChanged += textBox1_TextChanged;
            editor.KeyDown += textBox1_KeyDown;
            editor.PreviewKeyDown += textBox1_PreviewKeyDown;
        }


        public void ClearUndo() {
            while (editor.Undo()) {
            }            
        }

        protected void textBox1_PreviewKeyDown(object sender, KeyEventArgs e) {
            if (e.Key == Key.Tab) {
                int tabSize = 2;
                String tab = new String(' ', tabSize);
                int caretPosition = editor.CaretIndex;
                editor.Text = editor.Text.Insert(caretPosition, tab);
                editor.CaretIndex = caretPosition + tabSize;
                e.Handled = true;
            }
        }

        public void setLineNummer() {
            int charIndex = editor.CaretIndex;
            int lineindex = (editor.GetLineIndexFromCharacterIndex(charIndex) + 1);
            int index_aktLine = editor.GetCharacterIndexFromLineIndex(lineindex - 1);
            this.lRowNr.Content = "Zl: " + lineindex;
            int columnCount = editor.SelectionStart - index_aktLine + 1;
            this.lColumnNr.Content = "Sp: " + columnCount;
        }


        private void textBox1_KeyDown(object sender, KeyEventArgs e) {
            // setLineNummer();
        }

        private void textBox1_LostFocus(object sender, RoutedEventArgs e) {
            if (active == id) active = 0;
        }

        private void textBox1_GotFocus(object sender, RoutedEventArgs e) {
            active = id;
        }


        private void textBox1_TextChanged(object sender, TextChangedEventArgs e) {
            if (bEditor) {
                modified = true;
                if (isXMLEditor)
                    modell.EditXMLText = editor.Text;
                else
                    modell.EditXSDText = editor.Text;
            }
        }

        public void update() {
            loadFile();
        }

        public void redoClick() {
            editor.Redo();
        }
        public void undoClick() {
            editor.Undo();
        }


        public void loadFile() {
            if (isXMLEditor) {
                modell.loadFileXML();
                lDatei.Content = Basis.getFilename(modell.FilenameXML).Replace("_", "__");
                // lDatei.Content = "xsd__bsp1.xml";
                editor.Text = modell.EditXMLText;
                modell.XMLModified = false;  // change event des Editors wird aufgerufen
            }
            else {
                modell.loadFileXSD();
                lDatei.Content = Basis.getFilename(modell.FilenameXSD).Replace("_", "__");
                editor.Text = modell.EditXSDText;
                modell.XSDModified = false;  // change event des Editors wird aufgerufen
            }
        }


        public void Copy() {
            editor.Copy();
        }
        public void Cut() {
            editor.Cut();
        }

        public void Paste() {
            editor.Paste();
        }

        public void SelectAll() {
            editor.SelectAll();
        }

        public void searchText(String search, Boolean ignoreCase) {
            this.search = search;
            this.ignoreCase = ignoreCase;
            position = 0;
            int index;
            if (this.ignoreCase)
                index = editor.Text.IndexOf(search, position, StringComparison.OrdinalIgnoreCase);
            else
                index = editor.Text.IndexOf(search, position);
            position = index + 1;
            if (index > 0) {
                editor.Select(index, search.Length);
                editor.SelectionStart = index;
                // gescrollt wird alleine
                // editor.ScrollToLine( basis.getLinenr(index, editor.Text) );
                //editor.ScrollToHorizontalOffset(                
            }
            else {
                MessageBox.Show("Die Suche war nicht erfolgreich.\r\nSuchwert: " + search,
                    "Suchen", MessageBoxButton.OK, MessageBoxImage.Exclamation);
            }
        }

        public void searchAgain() {
            int index;
            if (this.ignoreCase)
                index = editor.Text.IndexOf(search, position, StringComparison.OrdinalIgnoreCase);
            else
                index = editor.Text.IndexOf(search, position);
            position = index + 1;
            if (index > 0) {
                editor.Select(index, search.Length);
                editor.SelectionStart = index;
                // gescrollt wird alleine
                // editor.ScrollToLine( basis.getLinenr(index, editor.Text) );
                //editor.ScrollToHorizontalOffset(                
            }
        }

        public void prettyPrintXML() {
            StringBuilder output = new StringBuilder();  // schneller als ein String                
            int tab = 0;
            int i1, i2;
            String line;
            String[] lines = Basis.getLines(editor.Text);
            for (int i = 0; i < lines.Count(); i++) {
                line = lines[i];
                i1 = line.IndexOf("<?");  // XML version
                if (i1 >= 0) {
                    output.AppendLine(line.Trim());
                }
                else {
                    if (line.IndexOf("<") < 0 && line.IndexOf("</") < 0) {
                        // zusatztext
                        output.AppendLine(line);
                    }
                    else {
                        // <name>bbbb</name>
                        // <student>
                        //   <name>
                          // Reihenfolge ist wichtig
                        i1 = line.IndexOf("<");
                        i2 = line.IndexOf("</");
                        if (i1 == i2) {
                              // testfall  </name>
                            output.AppendLine(getTabs(tab) + line.Trim());
                            tab--;
                        }
                        else {
                            // testfall  <name>xxx</name>  oder <name>
                            if (i1 >= 0) {
                                tab++;
                                output.AppendLine(getTabs(tab) + line.Trim());
                                if (i2 >= 0) {
                                    tab--;
                                }
                            }
                        }
                    }
                }
            }  // for (int i = 0; i < lines.Count(); i++)
            editor.Text = output.ToString();
            modified = true;
        }// prettyPrintXML


        // nicht benutzt, alte version
        public void prettyPrintXML2() {
            StringBuilder output = new StringBuilder();  // schneller als ein String                
            // Create an XmlReader
            String tab;
            if (SingVar.getInstance().FormatTab)
                tab = "\t";
            else
                tab = "  ";
            using (XmlReader reader = XmlReader.Create(new StringReader(editor.Text))) {
                XmlWriterSettings ws = new XmlWriterSettings();
                ws.Indent = true;           // wichtig
                ws.IndentChars = tab;      // wichtig
                using (XmlWriter writer = XmlWriter.Create(output, ws)) {
                    // Parse the file and display each of the nodes.
                    while (reader.Read()) {
                        switch (reader.NodeType) {
                            case XmlNodeType.Element:
                                writer.WriteStartElement(reader.Name);
                                break;
                            case XmlNodeType.Text:
                                writer.WriteString(reader.Value);
                                break;
                            case XmlNodeType.XmlDeclaration:
                            case XmlNodeType.ProcessingInstruction:
                                writer.WriteProcessingInstruction(reader.Name, reader.Value);
                                break;
                            case XmlNodeType.Comment:
                                writer.WriteComment(reader.Value);
                                break;
                            case XmlNodeType.EndElement:
                                writer.WriteFullEndElement();
                                break;
                        }
                    }

                }
            }
            editor.Text = output.ToString();
            modified = true;
        }  // prettyPrintxml

        private String getTabs(int tablength) {
            String tab;
            if (SingVar.getInstance().FormatTab)
                tab = "\t";
            else
                tab = "  ";

            String sErg = "";
            for (int i = 0; i < tablength; i++) {
                sErg += tab;
            }
            return sErg;
        }

        public void prettyPrintXSD() {
            StringBuilder output = new StringBuilder();  // schneller als ein String                
            int tab = 0;
            int i1, i2;
            String line;
            String[] lines = Basis.getLines(editor.Text);
            for (int i = 0; i < lines.Count(); i++) {
                line = lines[i];
                i1 = line.IndexOf("<?");
                if (i1 >= 0) {
                    output.AppendLine(line.Trim());
                }
                else {
                    i1 = line.IndexOf("</");  // Reihenfolge ist wichtig
                    if (i1 >= 0) {
                        tab--;
                        if (line.IndexOf("</xsd:schema>") >= 0) {
                            output.AppendLine("");
                        }
                        output.AppendLine(getTabs(tab) + line.Trim());
                        //if (tab == 1) output.AppendLine("");
                        
                    }
                    else {
                        i1 = line.IndexOf("<");
                        i2 = line.IndexOf("/>");  // Element in einer Zeile
                        if (i1 >= 0) {
                            if (line.IndexOf("<xsd:complexType")>=0  ||  line.IndexOf("<xsd:simpleType")>=0) {
                                // das 1. Element hat complexType, keine neue Zeile
                                if (i > 0 && lines[i - 1].IndexOf("<xsd:element") < 0) {
                                    output.AppendLine();
                                }
                            }
                            output.AppendLine(getTabs(tab) + line.Trim());
                            if (i2 < 0) {
                                tab++;
                            }
                        }
                    }
                }
            }  // for (int i = 0; i < lines.Count(); i++)
            editor.Text = output.ToString();
            modified = true;
        }// prettyPrintXSD




    }
}
